<?php

namespace Drupal\entity_add_another\Form;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\ContentEntityTypeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Entity Add Another settings for this site.
 */
class EntityAddAnotherSettingsForm extends ConfigFormBase implements ContainerInjectionInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The editable cloneable entities configuration entity.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->entityTypeManager = $container->get('entity_type.manager');
    $config_factory = $container->get('config.factory');
    $instance->config = $config_factory->getEditable('entity_add_another.add_another_entities');

    return $instance;

  }  

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['entity_add_another.add_another_entities'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'entity_add_another_add_another_entities_form';
  }  

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['#tree'] = TRUE;

    $form['add_another_entities'] = [
      '#tree' => TRUE,
      '#type' => 'fieldset',
      '#title' => $this->t('Add another entities'),
      '#description' => $this->t("
        Enable the add another functionalities for each entity type and bundle.
      "),
      '#open' => TRUE,
      '#collapsible' => FALSE,
    ];

    $form['add_another_entities']['table'] = [
      '#type' => 'table',
      '#header' => [
        'label' => $this->t('Label'),
        'default_value' => $this->t('Enabled'),
      ],
    ];

    $add_another_entities = $this->config->get('add_another_entities') ?? [];

    $entity_types = $this->entityTypeManager->getDefinitions();
    $types = array_filter($entity_types, function ($entity_type) {
      return $entity_type instanceof ContentEntityTypeInterface;
    });

    foreach ($types as $machine => $type) {
      $form['add_another_entities']['table'][$machine] = [
        'label' => [
          '#type' => 'label',
          '#title' => $this->t('@type', [
            '@type' => $type->getLabel(),
          ]),
        ],
        'value' => [
          '#type' => 'checkbox',
          '#default_value' => in_array($machine, $add_another_entities),
        ],
      ];
      $bundles = \Drupal::service('entity_type.bundle.info')->getBundleInfo($machine);
      if (!empty($bundles)) {
        foreach ($bundles as $key => $bundle) {
          $bundle_name = $bundle['label'];
          $type_and_bundle = $machine . '__' . $key;
          $form['add_another_entities']['table'][$type_and_bundle] = [
            'label' => [
              '#type' => 'label',
              '#title' => $this->t('@name', [
                '@name' => $type->getLabel() . ' of type ' . $bundle_name,
              ]),
            ],
            'value' => [
              '#type' => 'checkbox',
              '#default_value' => in_array($type_and_bundle, $add_another_entities),
            ],
          ];
        }
      }
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $add_another_entities = [];

    $settings = $form_state->getValue('add_another_entities');

    if (!isset($settings['table'])) {
      parent::submitForm($form, $form_state);
    }

    foreach ($settings['table'] as $name => $value) {
      if ($value['value'] != 1) {
        continue;
      }
      $add_another_entities[] = $name;
    }
    $this->config->set('add_another_entities', $add_another_entities)->save();

    parent::submitForm($form, $form_state);
  }

}
